
bl_info = {
    "name" : "Sync Render Sets",
    "author" : "CP Design", 
    "description" : "Synchronize all scene rendering settings and color spaces, and passes for all view layers",
    "blender" : (4, 2, 0),
    "version" : (1, 0, 0),
    "location" : "3Dview > N-Plane > Sync",
    "doc_url": "https://blendermarket.com/creators/cp-design", 
    "tracker_url": "", 
    "category" : "Render" 
}

import bpy
from bpy.types import Panel,Operator
if bpy.app.version >= (4, 1):
    from bpy.app.translations import pgettext_rpt as rpt_
else:
    from bpy.app.translations import pgettext_tip as rpt_

class SyncRenderPanels:
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Sync'

    @classmethod
    def poll(cls, context):
        return context.scene is not None 

class SYNC_PT_RenderColorPanel(SyncRenderPanels,Panel):
    bl_idname = "SYNC_PT_RenderColorPanel"
    bl_label = "Color Management"
    bl_order = 4  

    def draw(self, context):
        layout = self.layout

        layout.use_property_decorate = False  
        scenes = bpy.data.scenes
        panel=True
        for scene in scenes:
            scene_icon = "SCENE_DATA"  
            scene_txt = ""  
            if scene.name == bpy.context.scene.name:
                scene_icon = "FUND"  
                scene_txt = ""

            if bpy.app.version >= (4, 1):
                header, panel = layout.panel(scene.name+"my_panel_id", default_closed=False)
                header.label(text= f" {scene.name}" + scene_txt,icon=scene_icon)
                op=header.operator("sync.sync_render_sets",icon="UV_SYNC_SELECT")
                op.scene_index = scenes.find(scene.name)
                op.type = "Color"

            else:
                row = layout.row(align=True)
                row.label(text= f" {scene.name}" + scene_txt,icon=scene_icon)
                op=row.operator("sync.sync_render_sets",icon="UV_SYNC_SELECT")
                op.scene_index = scenes.find(scene.name)
                op.type = "Color"
            if panel:

                render_box = layout.box()
                flow = render_box.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=True)

                col = flow.column()
                col.prop(scene.display_settings, "display_device")

                col.separator()
                view = scene.view_settings
                col.prop(view, "view_transform")
                col.prop(view, "look")

                col = flow.column()
                col.prop(view, "exposure")
                col.prop(view, "gamma")

                col.separator()

                col.prop(scene.sequencer_colorspace_settings, "name", text="Sequencer")

                if bpy.app.version >= (4, 3):
                    r = render_box.row(align=True)
                    r.prop(view, "use_white_balance", text="White Balance")
                    if view.use_white_balance:
                        b=render_box.box()

                        if scene==context.scene:
                            eye = r.operator("ui.eyedropper_color", text="", icon='EYEDROPPER')
                            eye.prop_data_path = "scene.view_settings.white_balance_whitepoint"

                        col = b.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=False)
                        col.prop(view, "white_balance_temperature")
                        col.prop(view, "white_balance_tint")

                render_box.prop(scene.view_settings, "use_curve_mapping", text="Use Curves")
                if view.use_curve_mapping == True:
                    box=render_box.box()
                    box.template_curve_mapping(view, "curve_mapping", type='COLOR', levels=True)

            if bpy.app.version >= (4, 2):
                layout.separator(type="LINE")  
            else:
                layout.separator()

class SYNC_PT_LightPathsPanel(SyncRenderPanels,Panel):
    bl_idname = "SYNC_PT_LightPathsPanel"
    bl_label = "Light Paths"
    bl_order = 3  

    def draw(self, context):
        layout = self.layout
        scenes = bpy.data.scenes

        for scene in scenes:
            row = layout.row(align=True)
            col_label = row.column()
            col_label.scale_x = 1.5
            scene_icon = "SCENE_DATA"  
            scene_txt = ""  
            if scene.name == bpy.context.scene.name:
                scene_icon = "FUND"  
                scene_txt = ""
                col_label.alert = True 
            col_label.label(text= f" {scene.name}" + scene_txt, icon=scene_icon)  

            col0_prop = row.column()
            op=col0_prop.operator("sync.sync_render_sets",icon="UV_SYNC_SELECT")
            op.scene_index = scenes.find(scene.name)
            op.type = "Light Paths"

            renderset_box = layout.box()

            if scene.render.engine == 'CYCLES':
                ligthpaths_box = renderset_box.column()
                row = ligthpaths_box.row(align=True)

                col_label = row.column()
                col_label.scale_x = 1.0
                col_label.label(text='Light Paths')
                col_prop = row.column()           
                col_prop.prop(scene.cycles, "max_bounces", text="Total")

                row = ligthpaths_box.row(heading='', align=True)
                row.alignment = 'RIGHT'
                row.prop(scene.cycles, "diffuse_bounces", text="Diffuse")
                row.prop(scene.cycles, "glossy_bounces", text="Glossy")
                row.prop(scene.cycles, "transmission_bounces", text="Transmission")
                row.prop(scene.cycles, "volume_bounces", text="Volume")
                row.prop(scene.cycles, "transparent_max_bounces", text="Transparent")

                row = ligthpaths_box.row(heading='', align=True)
                row.prop(scene.cycles, "caustics_reflective", text="Refractive")
                row.prop(scene.cycles, "caustics_refractive", text="Refractive")

                row.prop(scene.cycles, "use_fast_gi", text="Fast GI Approximation")
                if scene.cycles.use_fast_gi:
                    row.prop(scene.cycles, "ao_bounces", text="Viewport Bounces")
                    row.prop(scene.cycles, "ao_bounces_render", text="Render Bounces")
            else :
                row = renderset_box.row(align=True)

                col_label = row.column()
                col_label.scale_x = 1.0
                col_label.label(text='Light Paths just in Cycles!', icon="ERROR")
                col0_prop = row.column()
                col0_prop.prop(scene.render, "engine", text="Render Engine")

            if bpy.app.version >= (4, 2):
                layout.separator(type="LINE")  
            else:
                layout.separator()

class SYNC_PT_SamplesPanel(SyncRenderPanels,Panel):
    bl_idname = "SYNC_PT_SamplesPanel"
    bl_label = "Samples"
    bl_order = 2  

    def draw(self, context):
        layout = self.layout
        scenes = bpy.data.scenes

        for scene in scenes:
            row = layout.row(align=True)
            col_label = row.column()
            col_label.scale_x = 1.5
            scene_icon = "SCENE_DATA"  
            scene_txt = ""  
            if scene.name == bpy.context.scene.name:
                scene_icon = "FUND"  
                scene_txt = ""
                col_label.alert = True 
            col_label.label(text= f" {scene.name}" + scene_txt, icon=scene_icon)  

            col = row.row(align=True)
            col.alignment = 'CENTER'

            col.prop(scene.render, "engine", text="")  
            if scene.render.engine == 'CYCLES':
                col.prop(scene.cycles, "feature_set", text="")
                col.prop(scene.cycles, "device", text="")  

            col.separator(factor=1.0)

            op=col.operator("sync.sync_render_sets",icon="UV_SYNC_SELECT", text="")
            op.scene_index = scenes.find(scene.name)
            op.type = "Samples"

            renderset_box = layout.box()
            renderset_box = renderset_box.column(align=True)            

            if scene.render.engine == 'CYCLES':
                sampling_box = renderset_box.box()
                row = sampling_box.row(align=True)

                col_label = row.column()
                col_label.scale_x = 1.0
                col_label.label(text='Render Samples')

                col_prop = row.column()
                col_prop.prop(scene.cycles, "use_adaptive_sampling", text="Noise Threshold")
                if scene.cycles.use_adaptive_sampling:
                    col1_prop = row.column()
                    col1_prop.prop(scene.cycles, "adaptive_threshold", text="")

                    row = sampling_box.row(heading='', align=True)
                    row.prop(scene.cycles, "samples", text="Max Samples")
                    row.prop(scene.cycles, "adaptive_min_samples", text="Min Samples")
                    row.prop(scene.cycles, "time_limit", text="Time Limit")

                else:
                    col1_prop = row.column()
                    col1_prop.label(text='Noise Threshold', icon='UNLINKED') 

                    row = sampling_box.row(heading='', align=True)
                    row.prop(scene.cycles, "samples", text="Samples")
                    row.prop(scene.cycles, "time_limit", text="Time Limit")

                if scene.cycles.feature_set == 'EXPERIMENTAL':
                    box=renderset_box.box()
                    row_2 = box.split(factor=0.07, align=True)
                    row_2.label(text='Subdivision')
                    cscene = scene.cycles
                    row_2.prop(cscene, "dicing_rate", text="Dicing Rate Render")
                    row_2.prop(cscene, "preview_dicing_rate", text="Viewport")

                    row_2.prop(cscene, "offscreen_dicing_scale", text="Offscreen Scale")
                    row_2.prop(cscene, "max_subdivisions")

                    renderset_box.separator(factor=0.5)

                row_1 = renderset_box.row(align=True)
                row_1.prop(scene.cycles, "use_denoising", text="Denoise" if scene.cycles.use_denoising else "Render Denoise" , toggle=1)
                if scene.cycles.use_denoising:
                    row_d = row_1.row(align=True)
                    row_d.prop(scene.cycles, "denoiser",text="")
                    row_d.prop(scene.cycles, "denoising_input_passes", text="")
                    row_d.prop(scene.cycles, "denoising_prefilter", text="")
                    row_d.prop(scene.cycles, "denoising_quality", text="")
                    row_d.prop(scene.cycles, "denoising_use_gpu", toggle=1)

                    renderset_box.separator(factor=0.5)

                row_t = renderset_box.row(align=False)
                row_t.prop(scene.render, "film_transparent", text="Film Transparent", toggle=1)
                if scene.render.film_transparent:
                    row_t2 = row_t.row(align=True)
                    row_t2.enabled = scene.render.film_transparent
                    row_t2.prop(scene.cycles, "film_transparent_glass", text="Transparent Glass")  
                    row_t2 = row_t.row(align=True)
                    row_t2.enabled = scene.render.film_transparent and scene.cycles.film_transparent_glass
                    row_t2.prop(scene.cycles, "film_transparent_roughness", text="Roughness Threshold")

                    renderset_box.separator(factor=0.5)

            elif scene.render.engine == 'BLENDER_EEVEE':

                row = renderset_box.row(align=True)

                row.prop(scene.eevee, "taa_render_samples", text="Render Samples")
                row.prop(scene.eevee, "taa_samples", text="Viewport Samples")
                row.prop(scene.eevee, "use_taa_reprojection", text="Viewport Denoising", toggle=1)

            elif scene.render.engine == 'BLENDER_EEVEE_NEXT':
                props = scene.eevee

                row = renderset_box.row(align=True)

                row.prop(props, "taa_render_samples", text="Render Samples")

                sp = renderset_box.split(factor=0.12, align=True)
                sp.label(text='Viewport')
                sp.prop(props, "taa_samples", text="Samples")
                sp.prop(props, "use_taa_reprojection",text="Temporal Reprojection", toggle=1)
                sp.prop(props, "use_shadow_jitter_viewport",text="Temporal Reprojection", toggle=1)

                col_shadows = renderset_box.column(heading="", align=True)
                col_shadows = col_shadows.box()
                col_shadows.prop(props, "use_shadows", text="Shadows", toggle=1)
                if props.use_shadows:
                    flow = col_shadows.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=True)        
                    flow.prop(props, "shadow_ray_count", text="Rays")
                    flow.prop(props, "shadow_step_count", text="Steps")
                    flow.prop(props, "shadow_resolution_scale", text="Resolution")

                    sub = flow.row(align=True)
                    sub.prop(props, "use_volumetric_shadows", text="Volume Shadows")
                    sub = sub.row(align=True)
                    sub.active = props.use_volumetric_shadows
                    sub.prop(props, "volumetric_shadow_samples", text="Steps")

                renderset_box.separator(factor=0.5)

                col_ray = renderset_box.column(heading="", align=True)
                col_ray = col_ray.box()
                col_ray.prop(props, "use_raytracing", text="Raytracing", toggle=1)
                if props.use_raytracing:
                    props = scene.eevee
                    options = scene.eevee.ray_tracing_options

                    flow = col_ray.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=True)        
                    flow.prop(props, "ray_tracing_method", text="Method")
                    flow.prop(options, "resolution_scale")

                    b_row = flow.box()
                    col = b_row.row(align=True)
                    col.label(text='Screen Tracing')
                    col.prop(options, "screen_trace_quality", text="Precision")
                    col.prop(options, "screen_trace_thickness", text="Thickness")

                    c_row = flow.box()
                    c_row.prop(options, "use_denoise", text="Denoising", toggle=1)
                    if options.use_denoise:
                        row_de = c_row.row(align=True)
                        row_de.prop(options, "denoise_spatial")

                        row_de1 = row_de.row(align=True)
                        row_de1.active = options.use_denoise and options.denoise_spatial
                        row_de1.prop(options, "denoise_temporal")

                        row_de2 = row_de1.row(align=True)
                        row_de2.active = options.use_denoise and options.denoise_spatial and options.denoise_temporal
                        row_de2.prop(options, "denoise_bilateral")

                    d_row = flow.box()
                    d_row.prop(props, "use_fast_gi", text="Fast GI Approximation", toggle=1)
                    if props.use_fast_gi:

                        flow_gi = d_row.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=True)        
                        flow_gi.prop(options, "trace_max_roughness", text="Threshold")
                        if props.ray_tracing_options.trace_max_roughness < 1:
                            flow_gi.prop(props, "fast_gi_method")
                            flow_gi.prop(props, "fast_gi_resolution", text="Resolution")
                            flow_gi.prop(props, "fast_gi_ray_count", text="Rays")
                            flow_gi.prop(props, "fast_gi_step_count", text="Steps")
                            flow_gi.prop(props, "fast_gi_quality", text="Precision")
                            flow_gi.prop(props, "fast_gi_distance")
                            flow_gi.prop(props, "fast_gi_thickness_near", text="Thickness Near")
                            flow_gi.prop(props, "fast_gi_thickness_far", text="Far")

                            flow_gi.prop(props, "fast_gi_bias", text="Bias")

            if bpy.app.version >= (4, 2):
                layout.separator(type="LINE")  
            else:
                layout.separator()

class SYNC_PT_ResolutionPanel(SyncRenderPanels,Panel):
    bl_idname = "SYNC_PT_ResolutionPanel"
    bl_label = "Resolution"
    bl_order = 1  

    def draw(self, context):
        layout = self.layout
        scenes = bpy.data.scenes

        for scene in scenes:

            rd = scene.render
            final_res_x = (rd.resolution_x * rd.resolution_percentage) / 100
            final_res_y = (rd.resolution_y * rd.resolution_percentage) / 100
            final_res_x_border = round(
                (final_res_x * (rd.border_max_x - rd.border_min_x)))
            final_res_y_border = round(
                (final_res_y * (rd.border_max_y - rd.border_min_y)))

            row = layout.row(align=True)
            col_label = row.column()
            col_label.scale_x = 1.5
            scene_icon = "SCENE_DATA"  
            scene_txt = ""  
            if scene.name == bpy.context.scene.name:
                scene_icon = "FUND"  
                scene_txt = ""
                col_label.alert = True 
            col_label.label(text= f" {scene.name}" +"[{} x {}]".format(str(final_res_x)[:-2], str(final_res_y)[:-2]) , icon=scene_icon)  

            col_prop = row.column()

            op=col_prop.operator("sync.sync_render_sets", text="",icon="UV_SYNC_SELECT")
            op.scene_index = scenes.find(scene.name)
            op.type = "Resolution"

            row_2CC4F = layout.box()  
            flow = row_2CC4F.grid_flow(row_major=True, columns=0, even_columns=False, even_rows=False, align=True)        
            flow.prop(scene.render, "resolution_x", text="X")   
            flow.prop(scene.render, "resolution_y", text="Y") 
            flow.prop(scene.render, "resolution_percentage", text="%")
            if bpy.app.version >= (4, 2):
                flow.prop(scene.render, "compositor_device", text="Compositor Device")

            if scene.render.engine == 'CYCLES':
                row2 = flow.split(factor=0.05, align=True)
                row2.prop(scene.cycles, "use_auto_tile",icon_only=True)
                if scene.cycles.use_auto_tile== False:
                    row2.active = False
                row2.prop(scene.cycles, "tile_size")

            if bpy.app.version >= (4, 2):
                layout.separator(type="LINE")  
            else:
                layout.separator()

def sync_properties(scene, source_scene, properties):
    for prop_name in properties:
        attr_parts = prop_name.split('.')

        if len(attr_parts) > 1:
            obj = getattr(scene, attr_parts[0], None)
            source_obj = getattr(source_scene, attr_parts[0], None)

            if obj is not None and source_obj is not None:
                attr_name = attr_parts[1]
                if hasattr(obj, attr_name) and hasattr(source_obj, attr_name):
                    setattr(obj, attr_name, getattr(source_obj, attr_name))

class SYNC_OT_SyncRenderSets(Operator):
    bl_idname = "sync.sync_render_sets"
    bl_label = "Sync"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO", "INTERNAL"}

    scene_index: bpy.props.IntProperty(options={'HIDDEN', 'SKIP_SAVE'})
    type: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})

    @classmethod
    def description(cls, context, properties):
        scenename = bpy.data.scenes[properties.scene_index].name
        if properties.type == 'Light Paths':
            return rpt_("Synchronize the light paths of scene (%s) to other scenes") % (scenename)
        elif properties.type == 'Color':
            return rpt_("Synchronize the color space of scene (%s) to other scenes") % (scenename)
        elif properties.type == 'Samples':
            return rpt_("Synchronize the samples of scene (%s) to other scenes") % (scenename)
        elif properties.type == 'Resolution':
            return rpt_("Synchronize the resolution of scene (%s) to other scenes") % (scenename)
        else:
            return ""

    def invoke(self, context, event):
        self.execute(context)
        return {'FINISHED'}

    def execute(self, context):
        scenes = bpy.data.scenes
        source_scene = scenes[self.scene_index]

        for index, scene in enumerate(scenes):
            if index != self.scene_index:

                if self.type == "Light Paths":
                    if source_scene.render.engine == 'CYCLES' and scene.render.engine == 'CYCLES':

                        scene.cycles.max_bounces = source_scene.cycles.max_bounces
                        scene.cycles.diffuse_bounces = source_scene.cycles.diffuse_bounces
                        scene.cycles.glossy_bounces = source_scene.cycles.glossy_bounces
                        scene.cycles.transmission_bounces = source_scene.cycles.transmission_bounces
                        scene.cycles.volume_bounces = source_scene.cycles.volume_bounces
                        scene.cycles.transparent_max_bounces = source_scene.cycles.transparent_max_bounces
                        scene.cycles.use_fast_gi = source_scene.cycles.use_fast_gi
                        scene.cycles.ao_bounces = source_scene.cycles.ao_bounces
                        scene.cycles.ao_bounces_render = source_scene.cycles.ao_bounces_render    
                        scene.cycles.caustics_reflective = source_scene.cycles.caustics_reflective
                        scene.cycles.caustics_refractive = source_scene.cycles.caustics_refractive

                elif self.type == "Color":
                    scene.display_settings.display_device=source_scene.display_settings.display_device
                    scene.view_settings.view_transform=source_scene.view_settings.view_transform
                    scene.view_settings.look=source_scene.view_settings.look
                    scene.view_settings.exposure=source_scene.view_settings.exposure
                    scene.view_settings.gamma=source_scene.view_settings.gamma
                    scene.sequencer_colorspace_settings.name=source_scene.sequencer_colorspace_settings.name
                    if bpy.app.version >= (4, 3):
                        scene.view_settings.use_white_balance=source_scene.view_settings.use_white_balance
                        scene.view_settings.white_balance_temperature=source_scene.view_settings.white_balance_temperature
                        scene.view_settings.white_balance_tint=source_scene.view_settings.white_balance_tint
                    scene.view_settings.use_curve_mapping=source_scene.view_settings.use_curve_mapping

                elif self.type == "Samples":

                    scene.render.engine = source_scene.render.engine
                    if source_scene.render.engine == 'CYCLES' and scene.render.engine == 'CYCLES':

                        samples_properties = [
                            'cycles.device',          
                            'cycles.use_adaptive_sampling',  
                            'cycles.adaptive_threshold',     
                            'cycles.samples',          
                            'cycles.adaptive_min_samples',  
                            'cycles.time_limit',       
                            'cycles.use_denoising',    
                            'render.film_transparent', 
                            'cycles.film_transparent_glass',  
                            'cycles.denoiser',         
                            'cycles.denoising_input_passes',   

                            'cycles.feature_set',
                            'cycles.dicing_rate',
                            'cycles.preview_dicing_rate',
                            'cycles.offscreen_dicing_scale',
                            'cycles.max_subdivisions',

                            'cycles.denoiser',
                            'cycles.denoising_input_passes',
                            'cycles.denoising_prefilter',
                            'cycles.denoising_quality',
                            'cycles.denoising_use_gpu',

                            'render.film_transparent',
                            'cycles.film_transparent_glass',
                            'cycles.film_transparent_roughness',

                        ]

                        sync_properties(scene, source_scene, samples_properties)

                    elif source_scene.render.engine == 'BLENDER_EEVEE' and scene.render.engine == 'BLENDER_EEVEE':

                        scene.eevee.use_taa_reprojection = source_scene.eevee.use_taa_reprojection  
                        scene.eevee.taa_render_samples = source_scene.eevee.taa_render_samples  
                        scene.eevee.taa_samples = source_scene.eevee.taa_samples  

                    elif source_scene.render.engine == 'BLENDER_EEVEE_NEXT' and scene.render.engine == 'BLENDER_EEVEE_NEXT':
                        samples_properties = [
                            'eevee.taa_render_samples',
                            'eevee.taa_samples',
                            'eevee.use_taa_reprojection',
                            'eevee.use_shadow_jitter_viewport',
                            'eevee.use_shadows',
                            'eevee.shadow_ray_count',
                            'eevee.shadow_step_count',
                            'eevee.shadow_resolution_scale',
                            'eevee.use_volumetric_shadows',
                            'eevee.volumetric_shadow_samples',
                            'eevee.use_raytracing',
                            'eevee.ray_tracing_method',

                            'eevee.use_fast_gi',
                            'eevee.fast_gi_method',
                            'eevee.fast_gi_resolution',
                            'eevee.fast_gi_ray_count',
                            'eevee.fast_gi_step_count',
                            'eevee.fast_gi_quality',
                            'eevee.fast_gi_distance',
                            'eevee.fast_gi_thickness_near',
                            'eevee.fast_gi_thickness_far',
                            'eevee.fast_gi_bias',
                        ]

                        sync_properties(scene, source_scene, samples_properties)

                        ray_samples_properties = [
                            'ray_tracing_options.resolution_scale',
                            'ray_tracing_options.screen_trace_quality',
                            'ray_tracing_options.screen_trace_thickness',
                            'ray_tracing_options.use_denoise',
                            'ray_tracing_options.denoise_spatial',
                            'ray_tracing_options.denoise_temporal',
                            'ray_tracing_options.denoise_bilateral',
                            'ray_tracing_options.trace_max_roughness',
                        ]

                        sync_properties(scene.eevee, source_scene.eevee, ray_samples_properties)

                elif self.type == "Resolution":
                    scene.render.resolution_x = source_scene.render.resolution_x
                    scene.render.resolution_y = source_scene.render.resolution_y
                    scene.render.resolution_percentage = source_scene.render.resolution_percentage
                    if source_scene.render.engine == 'CYCLES' and scene.render.engine == 'CYCLES':
                        scene.cycles.use_auto_tile = source_scene.cycles.use_auto_tile
                        scene.cycles.tile_size = source_scene.cycles.tile_size
                    if bpy.app.version >= (4, 2):
                        scene.render.compositor_device=source_scene.render.compositor_device

        return {'FINISHED'}

class SYNC_PT_PassesPanel(SyncRenderPanels,Panel):
    bl_idname = "SYNC_PT_PassesPanel"
    bl_label = "Passes"
    bl_order = 0  
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
            layout = self.layout
            scenes = bpy.data.scenes
            sorted_view_layers = []  

            active_scene = bpy.context.scene
            active_view_layer = bpy.context.view_layer

            for scene in bpy.data.scenes:
                for view_layer in scene.view_layers:
                    sorted_view_layers.append((scene, view_layer,scene.render.engine))

            for scene, view_layer,engine in sorted_view_layers:
                row = layout.row(align=True)
                col_label = row.column()
                col_label.scale_x = 1.5
                scene_icon = "RENDERLAYERS"  
                scene_txt = ""  

                if view_layer.name == bpy.context.view_layer.name and scene.name == bpy.context.scene.name:
                    scene_icon = "FUND"  
                    scene_txt = ""
                    col_label.alert = True 

                col_label.label(text=f"{scene.name}:" + f"{view_layer.name}" +scene_txt , icon=scene_icon)

                col_prop = row.row()

                op=col_prop.operator("sync.sync_viewlayer_passes", text="",icon="UV_SYNC_SELECT")
                op.scene_index = bpy.data.scenes.find(scene.name)
                op.layername = view_layer.name

                pass_box = layout.box()

                column = pass_box.column()

                if scene.render.engine == 'CYCLES':
                    row_1 = column.row(align=True)
                    row_1.prop(view_layer, "use_pass_combined")
                    row_1.prop(view_layer, "use_pass_z")
                    row_1.prop(view_layer, "use_pass_mist")
                    row_1.prop(view_layer, "use_pass_normal")
                    row_1.prop(view_layer, "use_pass_object_index")

                    row_2 = column.row(align=True)
                    cycles_view_layer = view_layer.cycles
                    row_2.prop(cycles_view_layer, "denoising_store_passes", text="Denoising Data")
                    row_2.prop(view_layer, "use_pass_position")
                    row_2.prop(view_layer, "use_pass_uv")
                    row_2.prop(view_layer, "use_pass_vector")
                    row_2.prop(view_layer, "use_pass_material_index")

                elif scene.render.engine == 'BLENDER_EEVEE':
                    row_1 = column.row(align=True)
                    row_1.prop(view_layer, "use_pass_combined")
                    row_1.prop(view_layer, "use_pass_z")
                    row_1.prop(view_layer, "use_pass_mist")
                    row_1.prop(view_layer, "use_pass_normal")

                elif scene.render.engine == 'BLENDER_EEVEE_NEXT':
                    row_1 = column.row(align=True)
                    row_1.prop(view_layer, "use_pass_combined")
                    row_1.prop(view_layer, "use_pass_z")
                    row_1.prop(view_layer, "use_pass_mist")
                    row_1.prop(view_layer, "use_pass_normal")
                    row_1.prop(view_layer, "use_pass_position")
                    sub = row_1.row(align=True)
                    sub.active = not scene.render.use_motion_blur
                    sub.prop(view_layer, "use_pass_vector")

                column = pass_box.column()

                if scene.render.engine == 'CYCLES':
                    row_1 = column.row(align=True)
                    row_1.label(text="Diffuse")
                    row_1.prop(view_layer, "use_pass_diffuse_direct", text="Direct")
                    row_1.prop(view_layer, "use_pass_diffuse_indirect", text="Indirect")
                    row_1.prop(view_layer, "use_pass_diffuse_color", text="Color")

                    row_2 = column.row(align=True) 
                    row_2.label(text="Glossy")
                    row_2.prop(view_layer, "use_pass_glossy_direct", text="Direct")
                    row_2.prop(view_layer, "use_pass_glossy_indirect", text="Indirect")
                    row_2.prop(view_layer, "use_pass_glossy_color", text="Color")

                    row_3 = column.row(align=True)               
                    row_3.label(text="Transmission")
                    row_3.prop(view_layer, "use_pass_transmission_direct", text="Direct")
                    row_3.prop(view_layer, "use_pass_transmission_indirect", text="Indirect")
                    row_3.prop(view_layer, "use_pass_transmission_color", text="Color")

                    row_4 = column.row(align=True)
                    row_4.label(text="Volume")
                    cycles_view_layer = view_layer.cycles
                    row_4.prop(cycles_view_layer, "use_pass_volume_direct", text="Direct")
                    row_4.prop(cycles_view_layer, "use_pass_volume_indirect", text="Indirect")

                    row_5 = column.row(align=True)
                    row_5.label(text="Other")
                    row_5.prop(view_layer, "use_pass_emit", text="Emission")
                    row_5.prop(view_layer, "use_pass_environment")
                    row_5.prop(view_layer, "use_pass_ambient_occlusion", text="Ambient Occlusion")
                    row_5.prop(cycles_view_layer, "use_pass_shadow_catcher")

                elif scene.render.engine == 'BLENDER_EEVEE':
                    row_1 = column.row(align=True)
                    row_1.label(text="Diffuse")
                    row_1.prop(view_layer, "use_pass_diffuse_direct", text="Light")
                    row_1.prop(view_layer, "use_pass_diffuse_color", text="Color")

                    row_2 = column.row(align=True) 
                    row_2.label(text="Specular")
                    row_2.prop(view_layer, "use_pass_glossy_direct", text="Light")
                    row_2.prop(view_layer, "use_pass_glossy_color", text="Color")

                    row_4 = column.row(align=True)
                    row_4.label(text="Volume")
                    row_4.prop(view_layer.eevee, "use_pass_volume_direct", text="Light")

                    row_5 = column.row(align=True)
                    row_5.label(text="Other")
                    row_5.prop(view_layer, "use_pass_emit", text="Emission")
                    row_5.prop(view_layer, "use_pass_environment")
                    row_5.prop(view_layer, "use_pass_shadow")
                    row_5.prop(view_layer, "use_pass_ambient_occlusion",text="Ambient Occlusion")

                    row_6 = column.row(align=True)
                    row_6.label(text="Effects")
                    if scene.eevee.use_bloom:
                        row_6.prop(view_layer.eevee, "use_pass_bloom", text="Bloom")
                    row_6.prop(view_layer.eevee, "use_pass_transparent")

                elif scene.render.engine == 'BLENDER_EEVEE_NEXT':
                    row_1 = column.row(align=True)

                    row_1.label(text="Diffuse")
                    row_1.prop(view_layer, "use_pass_diffuse_direct", text="Light")
                    row_1.prop(view_layer, "use_pass_diffuse_color", text="Color")

                    row_2 = column.row(align=True) 
                    row_2.label(text="Specular")
                    row_2.prop(view_layer, "use_pass_glossy_direct", text="Light")
                    row_2.prop(view_layer, "use_pass_glossy_color", text="Color")

                    row_4 = column.row(align=True)
                    row_4.label(text="Volume")
                    row_4.prop(view_layer.eevee, "use_pass_volume_direct", text="Light")

                    row_5 = column.row(align=True)
                    row_5.label(text="Other")
                    row_5.prop(view_layer, "use_pass_emit", text="Emission")
                    row_5.prop(view_layer, "use_pass_environment")
                    row_5.prop(view_layer, "use_pass_shadow")
                    row_5.prop(view_layer, "use_pass_ambient_occlusion", text="Ambient Occlusion")
                    row_5.prop(view_layer.eevee, "use_pass_transparent", text="Transparent")

                row_6 = column.row(align=True)
                row_6.label(text="Cryptomatte")
                row_6.prop(view_layer, "use_pass_cryptomatte_object", text="Object")
                row_6.prop(view_layer, "use_pass_cryptomatte_material", text="Material")
                row_6.prop(view_layer, "use_pass_cryptomatte_asset", text="Asset")

                if bpy.app.version >= (4, 2):
                    layout.separator(type="LINE")  
                else:
                    layout.separator()

class SYNC_OT_SyncPassesOperator(Operator):
    bl_idname = "sync.sync_viewlayer_passes"
    bl_label = "Sync Passes"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO", "INTERNAL"}

    scene_index: bpy.props.IntProperty(options={'HIDDEN', 'SKIP_SAVE'})
    layername: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})

    @classmethod
    def description(cls, context, properties):
        return rpt_("Synchronize [%s] Passes to other viewlayers") % (properties.layername)

    def execute(self, context):

        active_scene = bpy.data.scenes[self.scene_index]
        active_view_layer = active_scene.view_layers.get(self.layername)

        active_passes = [attr for attr in dir(active_view_layer) if attr.startswith("use_pass") and getattr(active_view_layer, attr)]

        active_evpasses = [attr for attr in dir(active_view_layer.eevee) if attr.startswith("use_pass") and getattr(active_view_layer.eevee, attr)]

        active_cypasses = [attr for attr in dir(active_view_layer.cycles) if attr.startswith("use_pass") and getattr(active_view_layer.cycles, attr)]

        for scene in bpy.data.scenes:
            for view_layer in scene.view_layers:
                if scene != active_scene or view_layer != active_view_layer:

                    for attr in dir(view_layer):
                        if attr.startswith("use_pass"):
                            setattr(view_layer, attr, attr in active_passes)

                    for attr in dir(view_layer.eevee):
                        if attr.startswith("use_pass"):
                            setattr(view_layer.eevee, attr, attr in active_evpasses)

                    for attr in dir(view_layer.cycles):
                        if attr.startswith("use_pass"):
                            setattr(view_layer.cycles, attr, attr in active_cypasses)

                    view_layer.cycles.denoising_store_passes=active_view_layer.cycles.denoising_store_passes

        return {'FINISHED'}

specific_dict = {
    ('Operator','Sync'): '同步', 
    ('*','Synchronize the light paths of scene (%s) to other scenes'): '同步场景(%s)的光程到其它场景',
    ('*','Synchronize the color space of scene (%s) to other scenes'): '同步场景(%s)的色彩空间到其它场景',
    ('*','Synchronize the samples of scene (%s) to other scenes'): '同步场景(%s)的采样值到其它场景',
    ('*','Synchronize the resolution of scene (%s) to other scenes'): '同步场景(%s)的分辨率到其它场景',

    ('Operator','Sync Passes'): '同步视图层通道',   
    ('*','Synchronize [%s] Passes to other viewlayers'): '同步视图层(%s)通道状态到其它视图层',

    }

langs = {
    'zh_CN': specific_dict,
    'zh_HANS': specific_dict,
}

classes = (SYNC_PT_RenderColorPanel,
    SYNC_PT_LightPathsPanel,
    SYNC_OT_SyncRenderSets,
    SYNC_PT_SamplesPanel,
    SYNC_PT_ResolutionPanel, 
    SYNC_PT_PassesPanel,
    SYNC_OT_SyncPassesOperator,
    )

def register():
    for cls in classes:
        bpy.utils.register_class(cls)

    bpy.app.translations.register(__name__, langs)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    bpy.app.translations.unregister(__name__)

if __name__ == "__main__":
    register()
